(*********************************************************************)
(*                          Litmus                                   *)
(*                                                                   *)
(*        Luc Maranget, INRIA Paris-Rocquencourt, France.            *)
(*        Susmit Sarkar, University of Cambridge, UK.                *)
(*                                                                   *)
(*  Copyright 2010 Institut National de Recherche en Informatique et *)
(*  en Automatique and the authors. All rights reserved.             *)
(*  This file is distributed  under the terms of the Lesser GNU      *)
(*  General Public License.                                          *)
(*********************************************************************)

{
open Globals

exception Error of string

let set_intkm set arg =
  match Misc.string_of_intkm arg with
  | Some  x -> set x
  | None ->
      raise (Error "int[kM] parameter expected")

let set_int set arg =
  let x =
    try int_of_string arg
    with Failure _ ->  raise (Error "int parameter expected") in
  set x

let set_bool set arg =
  let x =
    try bool_of_string arg
    with Failure _ ->  raise (Error "bool parameter expected") in
  set x


}

let blank = [' ''\t''\r']
let not_blank = [^' ''\t''\n''\r']
let arg = (blank* '='? blank* (not_blank [^'\n']* as arg) blank* '\n')

rule main = parse
| eof
    { () }
| '#' [^'\n']* '\n'
| blank* '\n'
    { main lexbuf }
| ("gcc_opts"|"ccopts") arg 
    { set_gcc_opts arg ; main lexbuf }
| "gcc" arg 
    { set_gcc arg ; main lexbuf }
| "linkopt" arg
    { set_link_opt arg ; main lexbuf }
| "gas" arg
    { set_bool set_gas arg ; main lexbuf }
| "barrier" arg
    { set_barrier arg ; main lexbuf }
| "delay" arg
    { set_int set_delay arg ; main lexbuf }
| "launch" arg
    { set_launch arg ; main lexbuf }
| "size_of_test" arg
    { set_intkm set_size arg ; main lexbuf }
| "loop" arg
    { set_intkm set_time_loop arg ; main lexbuf }
| "limit" arg 
    { set_bool set_limit arg ; main lexbuf }
| "no" arg 
    { set_no arg ; main lexbuf }
| "hint" arg 
    { set_hint arg ; main lexbuf }
| "affinity" arg
    { set_affinity arg ; main lexbuf }
| "procs" arg
    { set_logical_procs arg ; main lexbuf }
| "force_affinity" arg
    { set_bool set_force_affinity arg ; main lexbuf }
| "cache" arg
   { set_bool set_cache arg ; main lexbuf }
| "number_of_run" arg
    { set_intkm set_run arg ; main lexbuf }
| ("memory"|"mem") arg
   { set_memory arg ; main lexbuf }
| "contiguous" arg
    { set_bool set_contiguous arg ; main lexbuf ; }
| "safer" arg
   { set_safer arg ; main lexbuf }
| "preload" arg
   { set_preload arg ; main lexbuf }
| "collect" arg
   { set_collect arg ; main lexbuf }
| "prealloc" arg
   { set_bool set_prealloc arg ; main lexbuf }
| "os" arg
   { set_os arg ; main lexbuf }
| ("word"|"ws") arg
  { set_word arg ; main lexbuf }
| "avail" arg
  { set_int set_avail arg ; main lexbuf }
| ("st"|"stride") arg
  { set_int set_stride arg ; main lexbuf }
| ("vb"|"verbose_barrier") arg
   { set_bool set_verbose_barrier arg ; main lexbuf }
| "isync" arg 
   { set_bool set_isync arg ; main lexbuf }
| "smt_mode" arg
   { set_smt_mode arg ; main lexbuf }
| "smt" arg
   { set_int set_smt arg ; main lexbuf }
| "speedcheck" arg
   { set_speedcheck arg ; main lexbuf }
| [^'\n']* as lxm '\n'?
  { raise (Error (Printf.sprintf "format: %s" lxm)) }
{

let lex chan = main (Lexing.from_channel chan)

}
